﻿using ProjectGasMachine.CollectionGenericObjects;
using ProjectGasMachine.Drawnings;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;

namespace ProjectGasMachine
{
    /// <summary>
    /// Форма работы с компанией и ее коллекцией
    /// </summary>
    public partial class FormGasMachineCollection : Form
    {
        /// <summary>
        /// хранение коллекций
        /// </summary>
        private readonly StorageCollection<DrawningMachine> _storageCollection;

        /// <summary>
        /// Компания
        /// </summary>
        private AbstractCompany? _company = null;

        /// <summary>
        /// конструктор
        /// </summary>
        public FormGasMachineCollection()
        {
            InitializeComponent();
            _storageCollection = new();
        }

        /// <summary>
        /// Выбор компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }
        /// <summary>
        /// Добавление газовоза в коллекцию
        /// </summary>
        /// <param name="machine"></param>
        private void SetMachine(DrawningMachine? machine)
        {
            if (_company == null || machine == null)
            {
                return;
            }

            if (_company + machine != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }




        /// <summary>
        /// добавление грузовика
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddMachine_Click(object sender, EventArgs e) 
        {
            FormGasConfig form = new();
            // TODO передать метод +
            form.AddEvent(SetMachine);
            form.Show();
        }

        /// <summary>
        /// удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveMachine_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
            {
                return;
            }


            if (MessageBox.Show("Удалить объект", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        /// <summary>
        /// Передача объекта в другую форму
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGoToCheck_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            DrawningMachine? machine = null;
            int counter = 100;

            while (machine == null)
            {
                machine = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }

            if (machine == null)
            {
                return;
            }

            FormGasMachine form = new()
            {
                SetMachine = machine
            };
            form.ShowDialog();
        }

        /// <summary>
        /// Перерисовка коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            pictureBox.Image = _company.Show();
        }

        /// <summary>
        /// Добавление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }

            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Удаление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCollectionDel_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            if (MessageBox.Show("Вы уверены, что хотите удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RerfreshListBoxItems();
        }

        /// <summary>
        /// Обновление списка в listBoxCollection
        /// </summary>
        private void RerfreshListBoxItems()
        {
            listBoxCollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i];
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxCollection.Items.Add(colName);
                }
            }
        }

        /// <summary>
        /// Создание компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            ICollectionGenericObjects<DrawningMachine>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new Autopark(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }
            panelCompanyTools.Enabled = true;
            RerfreshListBoxItems();
        }
    }
}
